Package: mosum
Title: Moving Sum Based Procedures for Changes in the Mean
Version: 1.2.7
Date: 2022-10-20
Authors@R: c(
  person("Alexander", "Meier", email = "meier.alexander@posteo.de", role="aut"),
  person("Haeran", "Cho", email = "haeran.cho@bristol.ac.uk", role=c("aut", "cre")),
  person("Claudia", "Kirch", email = "claudia.kirch@ovgu.de", role="aut"))
Description: Implementations of MOSUM-based statistical procedures and algorithms for detecting multiple changes in the mean. This comprises the MOSUM procedure for estimating multiple mean changes from Eichinger and Kirch (2018) <doi:10.3150/16-BEJ887> and the multiscale algorithmic extension from Cho and Kirch (2022) <doi:10.1007/s10463-021-00811-5>, as well as the bootstrap procedure for generating confidence intervals about the locations of change points as proposed in Cho and Kirch (2022) <doi:10.1016/j.csda.2022.107552>. See also Meier, Kirch and Cho (2021) <doi:10.18637/jss.v097.i08> which accompanies the R package.
Depends: R (>= 3.1.2)
License: GPL (>= 3)
Imports: methods, RColorBrewer, plot3D, Rcpp (>= 0.12.5)
LinkingTo: Rcpp
Maintainer: Haeran Cho <haeran.cho@bristol.ac.uk>
RoxygenNote: 7.1.1
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2022-10-20 20:42:44 UTC; mahrc
Author: Alexander Meier [aut],
  Haeran Cho [aut, cre],
  Claudia Kirch [aut]
Repository: CRAN
Date/Publication: 2022-10-22 12:52:36 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 11:01:59 UTC; unix
Archs: mosum.so.dSYM
