# Generated by roxygen2: do not edit by hand

S3method(confint,mosum.cpts)
S3method(confint,multiscale.cpts)
S3method(plot,mosum.cpts)
S3method(plot,multiscale.cpts)
S3method(print,mosum.cpts)
S3method(print,multiscale.cpts)
S3method(summary,mosum.cpts)
S3method(summary,multiscale.cpts)
export(bandwidths.default)
export(mosum)
export(mosum.criticalValue)
export(multiscale.bottomUp)
export(multiscale.localPrune)
export(persp3D.multiscaleMosum)
export(testData)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(methods,is)
importFrom(plot3D,persp3D)
importFrom(stats,confint)
importFrom(stats,plot.ts)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,time)
useDynLib(mosum, .registration = TRUE)
